@@ -24,7 +24,7 @@ class CControllerProfileUpdate extends CController {
 	protected function checkInput() {
 		$fields = [
 			'idx' =>		'required|string',
-			'value_int' =>	'required|in 0,1',
+			'value_int' =>	'required|int32',
 			'idx2' =>		'array_id'
 		];
 
@@ -65,11 +65,15 @@ class CControllerProfileUpdate extends CController {
 				case 'web.sysmapconf.filter.state':
 				case 'web.user.filter.state':
 				case 'web.usergroup.filter.state':
+				case 'web.dashbrd.filter.state':
 					$ret = true;
 					break;
 
 				case 'web.latest.toggle':
 				case 'web.latest.toggle_other':
+				case 'web.dashbrd.timelinefixed':
+				case 'web.dashbrd.navtree.item.selected':
+				case !!preg_match('/web.dashbrd.navtree-\d+.toggle/', $this->getInput('idx')):
 					$ret = $this->hasInput('idx2');
 					break;
 
@@ -97,6 +101,8 @@ class CControllerProfileUpdate extends CController {
 		switch ($idx) {
 			case 'web.latest.toggle':
 			case 'web.latest.toggle_other':
+			case 'web.dashbrd.timelinefixed':
+			case !!preg_match('/web.dashbrd.navtree-\d+.toggle/', $this->getInput('idx')):
 				if ($value_int == 1) { // default value
 					CProfile::delete($idx, $this->getInput('idx2'));
 				}
@@ -107,6 +113,12 @@ class CControllerProfileUpdate extends CController {
 				}
 				break;
 
+			case 'web.dashbrd.navtree.item.selected':
+				foreach ($this->getInput('idx2') as $idx2) {
+					CProfile::update($idx, $value_int, PROFILE_TYPE_INT, $idx2);
+				}
+				break;
+
 			default:
 				if ($value_int == 1) { // default value
 					CProfile::delete($idx);
