@@ -67,7 +67,9 @@ switch ($data['method']) {
 		break;
 
 	case 'message.settings':
-		$result = getMessageSettings();
+		$msgsettings = getMessageSettings();
+		$msgsettings['timeout'] = timeUnitToSeconds($msgsettings['timeout']);
+		$result = $msgsettings;
 		break;
 
 	case 'message.get':
@@ -79,7 +81,7 @@ switch ($data['method']) {
 		}
 
 		// timeout
-		$timeout = time() - $msgsettings['timeout'];
+		$timeout = time() - timeUnitToSeconds($msgsettings['timeout']);
 		$lastMsgTime = 0;
 		if (isset($data['params']['messageLast']['events'])) {
 			$lastMsgTime = $data['params']['messageLast']['events']['time'];
@@ -169,7 +171,7 @@ switch ($data['method']) {
 		if (!CSession::keyExists('serverCheckResult')
 				|| (CSession::getValue('serverCheckTime') + SERVER_CHECK_INTERVAL) <= time()) {
 			$zabbixServer = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT, ZBX_SOCKET_TIMEOUT, 0);
-			CSession::setValue('serverCheckResult', $zabbixServer->isRunning());
+			CSession::setValue('serverCheckResult', $zabbixServer->isRunning(CWebUser::getSessionCookie()));
 			CSession::setValue('serverCheckTime', time());
 		}
 
@@ -196,6 +198,28 @@ switch ($data['method']) {
 		}
 		break;
 
+	case 'trigger.get':
+		$config = select_config();
+		$result = [];
+
+		$triggers = API::Trigger()->get([
+			'output' => ['triggerid', 'priority'],
+			'triggerids' => $data['triggerids'],
+			'limit' => $config['search_limit']
+		]);
+
+		if ($triggers) {
+			CArrayHelper::sort($triggers, [
+				['field' => 'priority', 'order' => ZBX_SORT_DOWN]
+			]);
+
+			foreach ($triggers as $trigger) {
+				$trigger['color'] = getSeverityColor($trigger['priority']);
+				$result[] = $trigger;
+			}
+		}
+		break;
+
 	/**
 	 * Create multi select data.
 	 * Supported objects: "applications", "hosts", "hostGroup", "templates", "triggers"
@@ -317,9 +341,14 @@ switch ($data['method']) {
 				break;
 
 			case 'triggers':
+				$host_fields = ['name'];
+				if (array_key_exists('real_hosts', $data) && $data['real_hosts']) {
+					$host_fields[] = 'status';
+				}
+
 				$triggers = API::Trigger()->get([
 					'output' => ['triggerid', 'description'],
-					'selectHosts' => ['name'],
+					'selectHosts' => $host_fields,
 					'editable' => isset($data['editable']) ? $data['editable'] : null,
 					'monitored' => isset($data['monitored']) ? $data['monitored'] : null,
 					'search' => isset($data['search']) ? ['description' => $data['search']] : null,
@@ -327,6 +356,18 @@ switch ($data['method']) {
 				]);
 
 				if ($triggers) {
+					if (array_key_exists('real_hosts', $data) && $data['real_hosts']) {
+						foreach ($triggers as $key => $trigger) {
+							foreach ($triggers[$key]['hosts'] as $host) {
+								if ($host['status'] != HOST_STATUS_MONITORED
+										&& $host['status'] != HOST_STATUS_NOT_MONITORED) {
+									unset($triggers[$key]);
+									break;
+								}
+							}
+						}
+					}
+
 					CArrayHelper::sort($triggers, [
 						['field' => 'description', 'order' => ZBX_SORT_UP]
 					]);
